%Created:       18.12.2012 by Peter Kandolf
%Last edit: 	18.12.2012 by Peter Kandolf
%Version:       0.1
%Author:        Peter Kandolf
%Remarks:
%
%Interface:
% [A,H,v] = MolenkampCrowley(DIMENSION)
%
% Computes an MolenkampCrowley operator of size DIMENSIONxDIMENSION with
% phase speed 1. It also returns a range of sugested step sizes as well as
% an initial vector. For more information on radial basis functions see
%
% G.E. Fassjaier. Meshfree approximation methods with MATLAB, volume 6 of
% Interdisciplinary Mathematical Sciences. World Scientif Publishing co.
% Pte. LTd., Hackensack, NJ, 2007.
%
%See also PHILEJA
%--------------------------------------------------------------------------
%Changes:
%   12.07.12 (PK):  changes in version 0
%                   file created
%   10.07.14 (PK):  added additional comments
function [A,k,v,f]=MolenkampCrowley(dimension)
N=dimension;
a=@(x,y)2*pi*(y);
b=@(x,y)-2*pi*(x);
rbf = @(r,e) exp(-(e*r).^2);
rbf_x = @(r,e,dx) -2*dx.*e.^2.*exp(-(e*r).^2);
rbf_y = @(r,e,dy) -2*dy.*e.^2.*exp(-(e*r).^2);
rbf_L = @(r,e) 2*e^2*exp(-(e*r).^2).*(2*(e*r).^2-1) ...
    -2*e^2*exp(-(e*r).^2);

I=[-1, -1;...
    1, 1];
[X,Y]=meshgrid(linspace(I(1,1),I(2,1),N),linspace(I(1,1),I(2,1),N));
int_boundary=~(X(:)==I(1,1) | X(:)==I(2,1) | Y(:)==I(2,2) | Y(:)==I(1,2));
%h=(I(2,1)-I(1,1))/N;
c=[1,1];%./(2*h);
center = [X(:),Y(:)];
f1=@(X,Y)1.1734*exp(-100*((X-0.2).^2+(Y-0.2).^2)).*((I(1,1)-X).^2.*(I(2,1)-X).^2.*(I(1,2)-Y).^2.*(I(2,2)-Y).^2);
e=getRBFFormParameter([dimension,dimension+3],f1,rbf,I)

r=DistanceMatrix(center,center);
t=sparse(rbf(r,e));
DR=sparse(repmat(c(1)*a(0,center(:,2)),1,size(center,1)).*...
    rbf_x(r,e,DifferenceMatrix(center,center,1))...
    +repmat(c(2)*b(center(:,1),0),1,size(center,1)).*...
    rbf_y(r,e,DifferenceMatrix(center,center,2)));
A=full((DR/t).*repmat(int_boundary,1,length(int_boundary)));
k=[1/2,1,1.5];

f=@(t)f1(-cos(2*pi*t)*X+sin(2*pi*t)*Y,-sin(2*pi*t)*X-cos(2*pi*t)*Y);
v=f(0);
v=v(:);%/norm(v(:),inf);

% Rewritten version of the function DistanceMatrix described in 
%
% G.E. Fassjaier. Meshfree approximation methods with MATLAB, volume 6 of
% Interdisciplinary Mathematical Sciences. World Scientif Publishing co.
% Pte. LTd., Hackensack, NJ, 2007.
% to save storage.
%
% Accumulate sum of squares of coordinate differences
% The ndgrid command produces two MxN matrices:
%   dr, consisting of N identical columns (each containing
%       the d-th coordinate of the M data sites)
%   cc, consisting of M identical rows (each containing
%       the d-th coordinate of the N centers)
function DM = DistanceMatrix(dsites,ctrs)
[M,s] = size(dsites); [N,s] = size(ctrs);
DM = zeros(M,N);

for i = 1:N
    points=ctrs(i,:);
    difference = dsites-points(ones(M,1),:);%repmat(ctrs(i,:),M,1);
    DM(:,i)=sqrt(sum(difference.^2,2));
    % tmp(tmp<1e-4)=0;
    %       %if abs(tmp)>1e-8
    %      DM(:,i) = tmp; 
    %       %end
end

%
%   for d=1:s
%      [dr,cc] = ndgrid(dsites(:,d),ctrs(:,d));
%      DM = DM + (dr-cc).^2;
%   end
%   DM = sqrt(DM);

% Rewritten version of the function DifferenceMatrix described in 
%
% G.E. Fassjaier. Meshfree approximation methods with MATLAB, volume 6 of
% Interdisciplinary Mathematical Sciences. World Scientif Publishing co.
% Pte. LTd., Hackensack, NJ, 2007.
% to save storage.
%
% DM = DifferenceMatrix(datacoord,centercoord)
% Forms the difference matrix of the n-th coordinate of two sets of points in R
% (some fixed coordinate of point in R^s)
function DM = DifferenceMatrix(dsites,ctrs,n)
DM=zeros(size(dsites,1),size(ctrs,1));
for i=1:size(ctrs,1)
   DM(:,i)=dsites(:,1)-ctrs(i,n); 
end
%[dr,cc] = ndgrid(dsites(:,n),ctrs(:,n));
%DM = dr-cc;


% Form factor selection for the RBF interpolation as suggested in 
%
% P. Kandolf, Innovative integrators for time dependent PDEs, Master’s
% thesis, University of Innsbruck 2011.
%
function [epsilon]=getRBFFormParameter(Neval,f,rbf,I)
[X,Y]=meshgrid(linspace(I(1,1),I(2,1),Neval(1)),linspace(I(1,1),I(2,1),Neval(1)));
[X1,Y1]=meshgrid(linspace(I(1,1),I(2,1),Neval(2)),linspace(I(1,1),I(2,1),Neval(2)));

condmax=1e+11;
condmin=1e3;

f0=f(X,Y);
f1=f(X1,Y1);
err=zeros(80,2);
IM=DistanceMatrix([X(:),Y(:)],[X(:),Y(:)]);
EM=DistanceMatrix([X1(:),Y1(:)],[X(:),Y(:)]);
e=linspace(5e+0,1e+2,80);
for i=1:length(e)
    tmp=rbf(IM,e(i));
    err(i,2)=condest(tmp);
    err(i,1)=norm(abs(f1(:)-rbf(EM,e(i))*(tmp\f0(:))),inf);%fun(e(i));
    fprintf('FF for %s> try %1.4e, err=%1.4e, condest=%1.4e\n','gauss',e(i),err(i,1),err(i,2));
end
I=err(:,2)<condmax & err(:,2)>condmin;
if sum(I)
    err=err(I,:);
    e=e(I);
else
    %warning('Condition of interplolation matrix very high!!!');
end
[err(:,1),I]=sort(err(:,1)); err(:,2)=err(I,2);
e=e(I);
[~,ind]=min(err(err(:,1)<1.5*err(1,1),2));
%[~,ind]=min(err(err(:,1)<10^ceil(log10(err(1,1))),2));
%[~,ind]=min(err(:,1));
epsilon=e(ind);